"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var recompose_1 = require("recompose");
var fp_1 = require("lodash/fp");
var Helpers_1 = require("./Helpers");
exports.DELAY_TIMEOUT = 200;
exports.withDelayedHandlers = function (handlers, options) {
    if (options === void 0) { options = {}; }
    return recompose_1.compose(recompose_1.defaultProps(__assign({ delayTimeout: exports.DELAY_TIMEOUT, enableDebounce: true, enableThrottle: true }, options)), recompose_1.withHandlers(handlers), recompose_1.withPropsOnChange(["delayTimeout", "enableDebounce", "enableThrottle"].concat(Object.keys(handlers)), function (_a) {
        var delayTimeout = _a.delayTimeout, enableDebounce = _a.enableDebounce, enableThrottle = _a.enableThrottle, props = __rest(_a, ["delayTimeout", "enableDebounce", "enableThrottle"]);
        return fp_1.pipe(fp_1.pick(Object.keys(handlers)), fp_1.mapValues(function (handler) {
            if (enableDebounce || enableThrottle) {
                var handlerDebounced_1 = enableDebounce && fp_1.debounce(delayTimeout)(function (callback) { return callback(); });
                var handlerThrottled_1 = enableThrottle && fp_1.throttle(delayTimeout)(function (callback) { return callback(); });
                var delayedHandler = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    var callback = handler.apply(void 0, args);
                    if (handlerDebounced_1) {
                        handlerDebounced_1(callback);
                    }
                    if (handlerThrottled_1) {
                        handlerThrottled_1(callback);
                    }
                };
                delayedHandler.cancel = function () {
                    if (handlerDebounced_1) {
                        handlerDebounced_1.cancel();
                    }
                    if (handlerThrottled_1) {
                        handlerThrottled_1.cancel();
                    }
                };
                return delayedHandler;
            }
            else {
                return function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return handler.apply(void 0, args)();
                };
            }
        }))(props);
    }), recompose_1.lifecycle({
        componentWillUnmount: function () {
            var _this = this;
            Object.keys(handlers).forEach(function (key) {
                if (_this.props && _this.props[key] && _this.props[key].cancel) {
                    _this.props[key].cancel();
                }
            });
        }
    }), Helpers_1.omitProps(["delayTimeout", "enableDebounce", "enableThrottle"]));
};
